/*
 * $Id: IterUtils.java 286 2005-12-03 01:07:16Z davidb $
 *
 * Copyright (c) 2005 VeriSign, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package se.rfc.unbound.iter;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;

import org.apache.log4j.Logger;
import org.xbill.DNS.Name;
import org.xbill.DNS.RRSIGRecord;
import org.xbill.DNS.Section;
import org.xbill.DNS.TextParseException;

import se.rfc.unbound.Host;
import se.rfc.unbound.SMessage;
import se.rfc.unbound.SRRset;
import se.rfc.unbound.Util;
import se.rfc.unbound.cache.DNSCache;

public class IterUtils
{

  private static Logger log = Logger.getLogger(IterUtils.class);

  /**
   * Load a particular DNSCache implementation.
   * 
   * @param className The name of the DNSCache implementation.
   * @return an uninitialized instance of the DNSCache.
   * 
   * @throws Exception If something goes wrong with loading the class.
   */
  public static DNSCache loadDNSCache(String className) throws Exception
  {
    if (className == null || className.equals(""))
      throw new IllegalArgumentException("No DNSCache class specified");

    Class c = null;

    c = Class.forName(className);

    Object o = null;

    o = c.newInstance();

    return (DNSCache) o;
  }

  /**
   * Populate the forward table from the config file.
   * 
   * @param s The main forward parameter. This is a list of zones to forward.
   * @param config The configuration object.
   */
  public static Map configureForwardZones(String s, Properties config)
  {
    String[] zones = s.split("\\s+");
    if (zones == null || zones.length == 0) return null;

    Host[] empty_host_array = new Host[0];
    Map result = null;

    for (int i = 0; i < zones.length; i++)
    {
      try
      {
        String z = zones[i];
        if (!z.endsWith(".")) z += ".";
        Name n = Name.fromString(z);

        String p = config.getProperty("dns.forward." + zones[i]);
        if (p == null)
        {
          // log.warn("No nameservers configured for forward zone " +
          // zones[i]);
          continue;
        }

        String[] addrstrs = p.split("\\s+");
        if (addrstrs == null || addrstrs.length == 0)
        {
          // log.warn("No nameservers configured for forward zone " +
          // zones[i]);
          continue;
        }
        List targetList = new ArrayList();
        for (int j = 0; j < addrstrs.length; j++)
        {
          try
          {
            InetAddress addr = InetAddress.getByName(addrstrs[j]);
            Host h = new Host(addr);
            targetList.add(h);
          }
          catch (UnknownHostException e)
          {
            // log.warn("Couldn't get address from forward zone nameserver "
            // + addrstrs[j]);
            continue;
          }
        }

        if (result == null) result = new HashMap();
        // log.debug("adding forward zone: " + n);
        Host[] targets = (Host[]) targetList.toArray(empty_host_array);
        result.put(n, targets);
      }
      catch (TextParseException e)
      {
        // log.warn("Failed to parse forward zone name: " + zones[i], e);
      }
    }

    return result;
  }

  /**
   * Given a (response) message, set the TTL of every RRset to a specific
   * value.
   * 
   * @param m The message.
   * @param ttl The TTL to trim to. If an RRset is signed, and this ttl is
   *          higher than origTTL, then the TTL will be set to 0.
   */
  public static void setPrivateTTL(SMessage m, long ttl)
  {
    for (int sec = Section.ANSWER; sec <= Section.ADDITIONAL; sec++)
    {
      List rrset_list = m.getSectionList(sec);
      for (Iterator i = rrset_list.iterator(); i.hasNext();)
      {
        SRRset rrset = (SRRset) i.next();
        long this_ttl = ttl;

        if (rrset.isSigned())
        {
          RRSIGRecord rrsig = (RRSIGRecord) rrset.firstSig();
          long orig_ttl = rrsig.getOrigTTL();
          if (this_ttl > orig_ttl)
          {
            this_ttl = 0;
          }
        }

        rrset.setTTL(this_ttl);
      }
    }
  }

  public static Set parseAddressSet(String s)
  {
    if (s == null) return null;

    String[] addrs = s.split("\\s+");

    if (addrs == null || addrs.length == 0) return null;

    Set result = new HashSet();

    for (int i = 0; i < addrs.length; i++)
    {
      try
      {
        InetAddress a = InetAddress.getByName(addrs[i]);
        result.add(a);
      }
      catch (UnknownHostException e)
      {
        log.debug("Cannot parse/resolve address: " + addrs[i]);
      }
    }

    if (result.size() == 0) return null;
    return result;
  }

  public static int[] parseTargetFetchPolicy(int maxDependencyDepth,
      String s)
  {
    int[] res = new int[maxDependencyDepth + 1];

    StringTokenizer st = new StringTokenizer(s);
    int def = st.hasMoreTokens() ? Util.parseInt(st.nextToken(), 0) : 0;
    
    for (int i = 0; i < res.length; i++)
    {
      res[i] = st.hasMoreTokens() ? Util.parseInt(st.nextToken(), def) : def;
    }
    return res;
  }
}
