package se.rfc.unbound.server;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.PropertyConfigurator;

/**
 * Command line parsing for the Server.
 * 
 * @author davidb
 */
class ServerCLIState
{
  public Options    opts;
  public Properties config;
  public boolean    debug;

  // etc...

  public ServerCLIState()
  {
    opts = new Options();

    // boolean options
    opts.addOption("h", "help", false, "Print this message.");
    opts.addOption("v", "verbose", false, "Increase verbosity.");

    OptionBuilder.hasArg();
    OptionBuilder.withLongOpt("config");
    OptionBuilder.withArgName("config_file");
    OptionBuilder.withDescription("overriding configuration file");
    opts.addOption(OptionBuilder.create('c'));
  }

  public void parseCommandLine(String[] args) throws ParseException
  {
    CommandLineParser cli_parser = new PosixParser();
    CommandLine cli = cli_parser.parse(opts, args);

    if (cli.hasOption('h')) usage();
    if (cli.hasOption('v')) debug = true;

    try
    {
      if (cli.getOptionValue('c') != null)
      {
        config = readConfiguration(cli.getOptionValue('c'));
      }
      else
      {
        System.err.println("-c/--config option is required.");
        System.exit(64);
      }
    }
    catch (IOException e)
    {
      throw new ParseException(e.getMessage());
    }

    setupLogging();

    if (debug)
    {
      Logger rootLogger = Logger.getRootLogger();
      rootLogger.setLevel(Level.DEBUG);
      rootLogger.debug("Debug logging on.");
    }

  }

  /**
   * Instantiate a configuration from the given Java properties file.
   * 
   * @param filename A valid Java properties file.
   * @return The parsed configuration.
   */
  public static Properties readConfiguration(String filename)
      throws IOException
  {
    Properties config = new Properties();
    InputStream file = new FileInputStream(filename);
    config.load(file);
    file.close();

    return config;
  }

  public boolean hasLog4jConfig(Properties cf)
  {
    for (Iterator i = cf.keySet().iterator(); i.hasNext();)
    {
      String key = (String) i.next();
      if (key.startsWith("log4j."))
      {
        return true;
      }
    }
    return false;
  }

  public void setupLogging()
  {
    String s = config.getProperty("log4j.config");
    if (s != null)
    {
      PropertyConfigurator.configure(s);
    }
    else if (hasLog4jConfig(config))
    {
      PropertyConfigurator.configure(config);
    }
    else
    {
      // use a shorter layout than the TTCCC layout
      Logger root = Logger.getRootLogger();
      root.addAppender(new ConsoleAppender(
          new PatternLayout("%d [%p] %m\n")));
    }
  }

  /**
   * This is a basic usage statement that generates the actual usage from
   * the command line options object, and then quits.
   * 
   * @param opts the command line options, generally created by the
   *          setupCLI() method.
   */
  private void usage()
  {
    HelpFormatter f = new HelpFormatter();

    PrintWriter out = new PrintWriter(System.err);

    // print our own usage statement:
    f.printHelp(out,
        75,
        "server.sh [..options..] qname [qtype] [qclass]",
        null,
        opts,
        HelpFormatter.DEFAULT_LEFT_PAD,
        HelpFormatter.DEFAULT_DESC_PAD,
        null);

    out.flush();
    System.exit(64);
  }
}